﻿using NLog.LayoutRenderers;
using ProjectStormTrooper.CollectionGenericObjects;
using ProjectStormTrooper.Exceptions;

/// <summary>
/// Параметризованный набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class ListGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Список объектов, которые храним
    /// </summary>
    private readonly List<T?> _collection;

    /// <summary>
    /// Максимально допустимое число объектов в списке
    /// </summary>
    private int _maxCount;

    public int MaxCount
    {
        get => _maxCount;
        set
        {
            if (value > 0)
            {
                _maxCount = value;
            }
        }
    }
    public CollectionType GetCollectionType => CollectionType.List;

    public int Count => _collection.Count;

    /// <summary>
    /// Конструктор
    /// </summary>
    public ListGenericObjects()
    {
        _collection = new();
    }

    public T? Get(int position)
    {
        if(position > _collection.Count || position < 0)
        {
            throw new PositionOutOfCollectionException();
        }
        if (_collection[position] == null)
        {
            throw new ObjectNotFoundException();
        }
        return _collection[position];
    }

    public bool Insert(T obj)
    {
        if (Count + 1 > MaxCount)
        {
            throw new CollectionOverflowException();
        }
        _collection.Add(obj);

        return true;
    }

    public bool Insert(T obj, int position)
    {
        if (_collection.Count + 1 < _maxCount) 
        {
            throw new CollectionOverflowException();
        }
        if (position > _collection.Count || position < 0)
        {
            throw new PositionOutOfCollectionException();
        }
        _collection.Insert(position, obj);
        return true;
    }

    public T Remove(int position)
    {
        if (position > _collection.Count || position < 0)
        {
            throw new PositionOutOfCollectionException();
        }
        T temp = _collection[position];
        _collection.RemoveAt(position);
        return temp;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Count; ++i)
        {
            yield return _collection[i];
        }
    }
}