﻿using System.DirectoryServices;
using ProjectStormTrooper.Drawnings;
using ProjectStormTrooper.Entities;

namespace ProjectStormTrooper.Drawnings;

public class DrawningStormTrooper : DrawningWarPlane
{




    /// <summary>
    /// Инициализация свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="rockets">Признак наличия ракет</param>
    /// <param name="bombs">Признак наличия бомб</param>



    public DrawningStormTrooper(int speed, double weight, Color bodyColor, Color colorOfHead, Color additionalColor, bool bombs, bool rockets)
    {
        EntityWarPlane = new EntityStormTrooper(speed, weight, bodyColor, colorOfHead, additionalColor, bombs, rockets);

    }
    /// <summary>
    /// Конструктор принимающий объект Entity
    /// </summary>
    public DrawningStormTrooper(EntityWarPlane? entityWarPlane)
    {
        if (entityWarPlane != null)
        {
            EntityWarPlane = entityWarPlane;
        }
    }


    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>

    public override void DrawTransport(Graphics g)
    {
        if (EntityWarPlane == null || EntityWarPlane is not EntityStormTrooper stormTrooper|| !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);
        Brush additionalBrush = new SolidBrush(stormTrooper.AdditionalColor);
        Brush mainBrush = new SolidBrush(EntityWarPlane.BodyColor);
        Brush headBrush = new SolidBrush(EntityWarPlane.ColorOfHead);
        Rectangle bodyOfFighter = new Rectangle(_startPosX.Value, _startPosY.Value + 40, 120, 20);

        int middleOfBody = (bodyOfFighter.X + bodyOfFighter.Right) / 2;
        base.DrawTransport(g);


        // ракеты
        if (stormTrooper.Rockets)
        {
            //верхняя ракета 
            int middleOfWingX = (2*middleOfBody + 30 ) /2;
            int middleOfTopWingY = (2 * bodyOfFighter.Top - 50) / 2;
            Point topLeftTopRocket = new Point(middleOfWingX - 1, middleOfTopWingY + 3);
            Point bottomLeftTopRocket = new Point(middleOfWingX + 6, middleOfTopWingY + 12);
            Point edgeOfTopRocket = new Point(middleOfWingX + 12, middleOfTopWingY + 6);
            Point[] headOfTopRocket = new Point[] { topLeftTopRocket, bottomLeftTopRocket, edgeOfTopRocket };
            g.DrawPolygon(pen, headOfTopRocket);
            g.FillPolygon(additionalBrush, headOfTopRocket);
            //нижняя ракета

            int middleOfBottomWingY = (2*bodyOfFighter.Top +  50) / 2;
            Point topLeftBottomRocket = new Point(middleOfWingX + 8, middleOfBottomWingY + 2);
            Point bottomLeftBottomRocket = new Point(middleOfWingX + 4, middleOfBottomWingY + 12);
            Point edgeOfBottomRocket = new Point(middleOfWingX + 16, middleOfBottomWingY + 6);
            Point[] headOfBottomRocket = new Point[] { topLeftBottomRocket, bottomLeftBottomRocket, edgeOfBottomRocket };
            g.DrawPolygon(pen, headOfBottomRocket);
            g.FillPolygon(additionalBrush, headOfBottomRocket);
        }
        // бомбы
        if (stormTrooper.Bombs)
        {
            Rectangle topBombBody = new Rectangle(middleOfBody + 40, bodyOfFighter.Top - 10, 10, 10);
            Rectangle bottomBombBody = new Rectangle(middleOfBody + 40, bodyOfFighter.Bottom, 10, 10);
            //голова верхней бомбы
            Point topTopBomb = new Point(topBombBody.Right + 1, topBombBody.Top);
            Point bottomTopBomb = new Point(topBombBody.Right + 1, topBombBody.Bottom);
            Point edgeTopBomb = new Point(topBombBody.Right + 10, (topBombBody.Top + topBombBody.Bottom) / 2);
            Point[] headOfTopBomb = new Point[] { topTopBomb, bottomTopBomb, edgeTopBomb };
            g.DrawPolygon(pen, headOfTopBomb);
            g.FillPolygon(additionalBrush, headOfTopBomb);
            Point topBottomBomb = new Point(bottomBombBody.Right + 1, bottomBombBody.Top);
            Point bottomBottomBomb = new Point(bottomBombBody.Right + 1, bottomBombBody.Bottom);
            Point edgeBottomBomb = new Point(bottomBombBody.Right + 10, (bottomBombBody.Top + bottomBombBody.Bottom) / 2);
            Point[] headOfBottomBomb = new Point[] { topBottomBomb, bottomBottomBomb, edgeBottomBomb };
            g.DrawPolygon(pen, headOfBottomBomb);
            g.FillPolygon(additionalBrush, headOfBottomBomb);
            g.DrawRectangle(pen, topBombBody);
            g.DrawRectangle(pen, bottomBombBody);
            g.FillRectangle(additionalBrush, topBombBody);
            g.FillRectangle(additionalBrush, bottomBombBody);
        }


    }
}
