﻿using ProjectStormTrooper.Entities;

namespace ProjectStormTrooper.Drawnings;

/// <summary>
/// Расширение для класса EntityWarPlane
/// </summary>
public static class ExtentionDrawningWarPlane
{
    /// <summary>
    /// Разделитель для записи информации по объекту в файл
    /// </summary>
    private static readonly string _separatorForObject = ":";

    /// <summary>
    /// Создание объекта из строки
    /// </summary>
    /// <param name="info">Строка с данными для создания объекта</param>
    /// <returns>Объект</returns>
    public static DrawningWarPlane? CreateDrawningWarPlane(this string info)
    {
        string[] strs = info.Split(_separatorForObject);
        EntityWarPlane? warPlane = EntityStormTrooper.CreateEntityStormTrooper(strs);
        if (warPlane != null)
        {
            return new DrawningStormTrooper(warPlane);
        }

        warPlane = EntityWarPlane.CreateEntityCar(strs);
        if (warPlane != null)
        {
            return new DrawningWarPlane(warPlane);
        }

        return null;
    }

    /// <summary>
    /// Получение данных для сохранения в файл
    /// </summary>
    /// <param name="drawningCar">Сохраняемый объект</param>
    /// <returns>Строка с данными по объекту</returns>
    public static string GetDataForSave(this DrawningWarPlane drawningWarPlane)
    {
        string[]? array = drawningWarPlane?.EntityWarPlane?.GetStringRepresentation();

        if (array == null)
        {
            return string.Empty;
        }

        return string.Join(_separatorForObject, array);
    }
}