﻿namespace ProjectStormTrooper.Entities;


public class EntityStormTrooper : EntityWarPlane
{

    /// <summary>
    /// Дополнительный цвет (для опциональных элементов)
    /// </summary>

    public Color AdditionalColor { get; private set; }

    /// <summary>
    /// Признак (опция) наличия бомб
    /// </summary>

    public bool Bombs { get; private set; }

    /// <summary>
    /// Признак (опция) наличия ракет
    /// </summary>

    public bool Rockets { get; private set; }

    public void SetAdditionalColor(Color additionalColor)
    {
        AdditionalColor = additionalColor;
    }
    /// <summary>
    /// Инициализация полей объекта-класса штурмовика
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес штурмовика</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="bombs">Признак наличия бомб</param>
    /// <param name="rockets">Признак наличия ракет</param>

    public EntityStormTrooper(int speed, double weight, Color bodyColor, Color colorOfHead, Color additionalColor, bool bombs, bool rockets) : base(speed, weight, bodyColor, colorOfHead)
    {
  
        AdditionalColor = additionalColor;
        Bombs = bombs;
        Rockets = rockets;

    }
    /// <summary>
    /// Получение строк со значениями свойств объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public override string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityStormTrooper), Speed.ToString(), Weight.ToString(), BodyColor.Name, 
            ColorOfHead.Name, AdditionalColor.Name,Bombs.ToString(), Rockets.ToString() };
    }

    /// <summary>
    /// Создание объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityStormTrooper? CreateEntityStormTrooper(string[] strs)
    {
        if (strs.Length != 8 || strs[0] != nameof(EntityStormTrooper))
        {
            return null;
        }

        return new EntityStormTrooper(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]), Color.FromName(strs[4]),
            Color.FromName(strs[5]), Convert.ToBoolean(strs[6]), Convert.ToBoolean(strs[7]));
    }
}