﻿using ProjectStormTrooper.CollectionGenericObjects;
using ProjectStormTrooper.Drawnings;
using Microsoft.Extensions.Logging;
using ProjectStormTrooper.Exceptions;
using System.Xml.Linq;

namespace ProjectStormTrooper
{
    public partial class FormWarPlaneCollection : Form
    {
        private AbstractCompany? _company = null;
        private readonly StorageCollection<DrawningWarPlane> _storageCollection;
        /// <summary>
        /// Логер
        /// </summary>
        private readonly ILogger _logger;
        public FormWarPlaneCollection(ILogger<FormWarPlaneCollection> logger)
        {
            InitializeComponent();
            _storageCollection = new();
            _logger = logger;
        }
        /// <summary>
        /// Выбор компании
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
        {
            panelCompanyTools.Enabled = false;
        }
        /// <summary>
        /// Добавление обычного автомобиля
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddWarPlane_Click(object sender, EventArgs e)
        {
            FormWarPlaneConfig form = new FormWarPlaneConfig();
            form.AddEvent(SetWarPlane);
            form.Show();
        }

        /// <summary>
        /// Добавление автомобиля в коллекцию
        /// </summary>
        /// <param name="warPlane"></param>
        private void SetWarPlane(DrawningWarPlane? warPlane)
        {
            if (_company == null || warPlane == null)
            {
                return;
            }
            try
            {
                int addingObject = (_company + warPlane);
                MessageBox.Show("Объект добавлен");
                _logger.LogInformation($"Добавлен объект {warPlane.GetDataForSave()}");
                pictureBox.Image = _company.Show();
            }
            catch(CollectionOverflowException ex) 
            {
                MessageBox.Show("Не удалось добавить объект");
                _logger.LogWarning($"Не удалось добавить объект: {ex.Message}");

            }
            
        }

        /// <summary>
        /// Получение цвета
        /// </summary>
        /// <param name="random">Генератор случайных чисел</param>
        /// <returns></returns>
        private static Color GetColor(Random random)
        {
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }

            return color;
        }
        /// <summary>
        /// Удаление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveWarPlane_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text) || _company == null)
            {
                _logger.LogWarning("Удаление объекта из несуществующей коллекции");
                return;
            }

            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
            {
                return;
            }

            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            try
            {
                object decrementObject = _company - pos;
                MessageBox.Show("Объект удален");
                _logger.LogInformation($"Удален по позиции {pos}");
                pictureBox.Image = _company.Show();


            }
            catch(ObjectNotFoundException ex) 
            {
                MessageBox.Show("Объект не найден");
                _logger.LogWarning($"Удаление не найденного объекта в позиции {pos} ");
            }
            catch(PositionOutOfCollectionException ex) 
            {
                MessageBox.Show("Удаление вне рамках коллекции");
                _logger.LogWarning($"Удаление объекта за пределами коллекции {pos} ");
            }
        }
        /// <summary>
        /// Передача объекта в другую форму
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonGoToCheck_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            DrawningWarPlane? warPlane = null;
            int counter = 100;
            while (warPlane == null)
            {
                try
                {
                    warPlane = _company.GetRandomObject();
                }
                catch (ObjectNotFoundException)
                {
                    counter--;
                    if (counter <= 0)
                    {
                        break;
                    }
                }

            }

            if (warPlane == null)
            {
                return;
            }

            FormStormTrooper form = new()
            {
                SetWarPlane = warPlane
            };
            form.ShowDialog();
        }

        /// <summary>
        /// Перерисовка коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRefresh_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            pictureBox.Image = _company.Show();
        }
        /// <summary>
        /// Добавление коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCollectionAdd_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogWarning("Не заполненная коллекция");
                return;
            }

            CollectionType collectionType = CollectionType.None;
            if (radioButtonMassive.Checked)
            {
                collectionType = CollectionType.Massive;
            }
            else if (radioButtonList.Checked)
            {
                collectionType = CollectionType.List;
            }

            _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
            _logger.LogInformation($"Добавлена коллекция: {textBoxCollectionName.Text}");
            RerfreshListBoxItems();
        }

        /// <summary>
        /// Удаление коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCollectionDel_Click(object sender, EventArgs e)
        {
            if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                _logger.LogWarning("Удаление невыбранной коллекции");
                return;
            }
            string name = listBoxCollection.SelectedItem.ToString() ?? string.Empty;

            if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
            {
                return;
            }
            _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
            _logger.LogInformation($"Удалена коллекция: {name}");
            RerfreshListBoxItems();
        }

        /// <summary>
        /// Обновление списка в listBoxCollection
        /// </summary>
        private void RerfreshListBoxItems()
        {
            listBoxCollection.Items.Clear();
            for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
            {
                string? colName = _storageCollection.Keys?[i];
                if (!string.IsNullOrEmpty(colName))
                {
                    listBoxCollection.Items.Add(colName);
                }
            }

        }

        /// <summary>
        /// Создание компании
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreateCompany_Click(object sender, EventArgs e)
        {
            if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                _logger.LogWarning("Создание компании невыбранной коллекции");
                return;
            }

            ICollectionGenericObjects<DrawningWarPlane>? collection = _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
            if (collection == null)
            {
                MessageBox.Show("Коллекция не проинициализирована");
                _logger.LogWarning("Не удалось инициализировать коллекцию");
                return;
            }

            switch (comboBoxSelectorCompany.Text)
            {
                case "Ангар":
                    _company = new Hangar(pictureBox.Width, pictureBox.Height, collection);
                    break;
            }

            panelCompanyTools.Enabled = true;

            RerfreshListBoxItems();
        }

        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storageCollection.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogError("Ошибка: {Message}", ex.Message);
                }
            }
        }

        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if(openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storageCollection.LoadData(openFileDialog.FileName);
                    MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation("Загрузка из файла: {filename}", saveFileDialog.FileName);
                    RerfreshListBoxItems();
                }
                catch(Exception ex) 
                {                                     
                    MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogError("Ошибка: {Message}", ex.Message);
                }

                
                

            }
        }
    }
}
