﻿
using ProjectStormTrooper.Drawnings;

namespace ProjectStormTrooper.MovementStrategy;

/// <summary>
/// Класс-реализация IMoveableObject с использованием DrawningCar
/// </summary>
public class MoveableCar : IMoveableObject
{
    /// <summary>
    /// Поле-объект класса DrawningCar или его наследника
    /// </summary>
    private readonly DrawningWarPlane? _warPlane = null;

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="warPlane">Объект класса DrawningWarPlane</param>
    public MoveableCar(DrawningWarPlane warPlane)
    {
        _warPlane = warPlane;
    }

    public ObjectParameters? GetObjectPosition
    {
        get
        {
            if (_warPlane == null || _warPlane.EntityWarPlane == null || !_warPlane.GetPosX.HasValue || !_warPlane.GetPosY.HasValue)
            {
                return null;
            }
            return new ObjectParameters(_warPlane.GetPosX.Value, _warPlane.GetPosY.Value, _warPlane.GetWidth, _warPlane.GetHeight);
        }
    }

    public int GetStep => (int)(_warPlane?.EntityWarPlane?.Step ?? 0);

    public bool TryMoveObject(MovementDirection direction)
    {
        if (_warPlane == null || _warPlane.EntityWarPlane == null)
        {
            return false;
        }

        return _warPlane.MoveTransport(GetDirectionType(direction));
    }

    /// <summary>
    /// Конвертация из MovementDirection в DirectionType
    /// </summary>
    /// <param name="direction">MovementDirection</param>
    /// <returns>DirectionType</returns>
    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,
        };
    }
}